import {SimpleGamesystem} from "../../gamesystems/SimpleGamesystem";
import {GameModel} from "../../GameModel";
import {TemplateType} from "../TemplateType";
import {ProductTemplateSystem} from "./ProductTemplateSystem";
import {ProductGamesystem} from "../../gamesystems/ProductGamesystem";

export class ProductTemplateCreator {
  static constructTemplateFromSimpleGamesystem(simpleGamesystem: SimpleGamesystem, gameModel: GameModel, templateType: TemplateType) {
    const productGamesystem = new ProductTemplateSystem(simpleGamesystem.componentName, simpleGamesystem.componentDescription, templateType);
    productGamesystem.constructHierarchyFromSimpleGamesystem(simpleGamesystem, gameModel);
    return productGamesystem;
  }

  static convertProductToTemplate(productGamesystem: ProductGamesystem, gameModel: GameModel, templateType: TemplateType) {
    const productTemplate = new ProductTemplateSystem(productGamesystem.componentName, productGamesystem.componentDescription, templateType);
    productTemplate.states = productGamesystem.states
    productTemplate.transitions = productGamesystem.transitions
    productTemplate.innerGamesystems = productGamesystem.innerGamesystems;
    productGamesystem.innerGamesystems.forEach(innerGamesystem => innerGamesystem.parentGamesystem = productTemplate);

    if(productGamesystem.parentGamesystem == undefined) {
      gameModel.removeGamesystem(productGamesystem)
      gameModel.gamesystems.push(productTemplate)
    } else {
      productTemplate.parentGamesystem = productGamesystem.parentGamesystem;
      productGamesystem.parentGamesystem.addChildGamesystem(productTemplate);
    }
    return productTemplate;
  }
}
